# NtpDate
# set time of QBoxHD according internet 
# may be necessary, if transponder received at startup has wrong settings
# eDVBLocalTimeHandler (in \usr\local\lib\enigma2\python\enigma.py) uses the transponder time!
# Created 2010-12 by Horst Schmid, info@schmidhorst.de
# inspired from ... 
#   ntpdate-4.2.6p2_2-qboxhd.tar http://www.sat-linux.com/forum/showthread.php?t=12140
#   http://www.dream-multimedia-tv.de/board/thread.php?postid=41564
#   FritzCall, http://www.i-have-a-dreambox.com/wbb2/thread.php?threadid=100994
#   http://www.qboxsvn.com/projects/qboxhd/browser/e2-0.2/branches/enigma2_git/lib/python/Plugins/Extensions/FritzCall
#   http://www.i-have-a-dreambox.com/wbb2/thread.php?postid=1061366

# under http://www.dream-multimedia-tv.de/board/thread.php?threadid=8870 there is a similar
# tool for DreamBox available: enigma2-plugin-extensions-ntpupdatemanager_1.0_mipsel.ipk
# - a list with preset time servers for each country is used


# 2011-07-27 0.1.3 the message "Two sources (Transponder and Internet) are active!" poped up in wrong situation, fixed
#            line 'from qboxhd import QBOXHD' is not necessary and causes failure e.g. on DreamBox
#            Some conflict with EPG: with NtpDate EPG needs a GUI-Restart to work, if "useTransponderTime is off"!
#            Solution:  put the call of ntpdate to a shell file in /etc/rc.d instead of calling direct from here and
#            switch on 'useTransponderTime', which seems then to be done before the rc.d script.
#            (all files in /etc/rc.d are called from the end of /etc/init.d/rcS)

# 2012-08-01 0.1.4 EPG problem solved (use of /etc/rc.d was no solution!), eTimer added

g_pluginversion = "0.1.4" # 2012-08-04

from Plugins.Plugin import PluginDescriptor
from Components.ActionMap import ActionMap
from Components.config import config, ConfigSubsection, ConfigSelection, ConfigEnableDisable, getConfigListEntry, ConfigText, ConfigInteger
from Components.ConfigList import ConfigListScreen
from Components.Element import cached
from Components.Label import Label
from Components.Sources.StaticText import StaticText
from Screens.MessageBox import MessageBox
from Screens.Console import Console
from Screens.Screen import Screen
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Tools import Notifications
from os import popen, remove
from time import strftime, gmtime, localtime
from enigma import eTimer, eDVBLocalTimeHandler

#############
# Globals
g_readme = "/readme.txt"


config.plugins.ntpdate = ConfigSubsection()
config.plugins.ntpdate.enable = ConfigEnableDisable(default = True)
config.plugins.ntpdate.notify = ConfigEnableDisable(default = False)
config.plugins.ntpdate.showtime = ConfigInteger(default = 10, limits = (0,99))
config.plugins.ntpdate.timeout = ConfigInteger(default = 20, limits = (3,120))
config.plugins.ntpdate.delay1 = ConfigInteger(default = 0, limits = (0,600))
config.plugins.ntpdate.repeat = ConfigInteger(default = 1, limits = (0,168))
config.plugins.ntpdate.timeserver1 = ConfigText(default = "ntp.dianacht.de", fixed_size = False)
config.plugins.ntpdate.timeserver2 = ConfigText(default = "time.nuri.net", fixed_size = False)
config.plugins.ntpdate.timeserver3 = ConfigText(default = "swisstime.ethz.ch", fixed_size = False)

ntp_call = None

# http://www.dream-multimedia-tv.de/board/index.php?page=Thread&threadID=12337
# http://dream.reichholf.net/pydoc/html/d9/d4d/classenigma_1_1eTimer.html#aa31bcbc915412bb8be5d378515a83b7c
class MyTimer():
	def timed(self):
		before = "Before: Local=" + strftime("%H:%M", localtime()) +", UTC=" + strftime("%H:%M", gmtime())
		cmd = "%s%s %s %s %s %s" % (plugin_path, "/ntpdate -b -t ", config.plugins.ntpdate.timeout.value, config.plugins.ntpdate.timeserver1.value, config.plugins.ntpdate.timeserver2.value, config.plugins.ntpdate.timeserver3.value)
		res = popen(cmd).read()
		print "[ntpdate] via timer done: " + res
		if config.plugins.ntpdate.notify.value == True:
			Notifications.AddNotification(MessageBox, before + "\n\nntpdate " + res + "\nAfter: Local=" + strftime("%H:%M", localtime()) +", UTC=" + strftime("%H:%M", gmtime()) , type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)
		try:
			eDVBLocalTimeHandler.getInstance().setUseDVBTime(True) # tell EPG that time is correct
		except:
			print "[ntpdate] eDVBLocalTimeHandler.getInstance().setUseDVBTime(True) failed"
		x = config.plugins.ntpdate.repeat.value # Unit for value: hours
		if (x>0):
			print "[ntpdate] - next Sync via Timer in [h]:", x
			mytimer.start(x*3600, True)
			print "[ntpdate] timer set to [h]: ", x


	def __init__(self):
		self.timer = eTimer()
		self.timer.callback.append(self.timed)

	def start(self, time_s, single):
		self.timer.start(1000*time_s, single)
#		Notifications.AddNotification(MessageBox, "timer started" , type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)
		print "[ntpdate] timer start, [s]", time_s

	def stop(self):
		try:
			self.timer.stop()
			print "[ntpdate] timer stopped"
		except:
			print "[ntpdate] timer stopp failed"
#			Notifications.AddNotification(MessageBox, "timer stop failed! not started?" , type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)

	def savesession(self,session):
		self.session = session
		print "[ntpdate] savesession done"
#		Notifications.AddNotification(MessageBox, "session saved" , type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)

#end class MyTimer

mytimer = MyTimer()

class ntpdateSetup(ConfigListScreen, Screen):
  
	skin = """
		<screen position="100,90" size="550,400" title="ntpdate Setup (time from a network server)" >
		<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" transparent="1" alphatest="on" />
		<ePixmap pixmap="skin_default/buttons/green.png" position="150,0" size="140,40" transparent="1" alphatest="on" />
		<ePixmap pixmap="skin_default/buttons/yellow.png" position="300,0" size="140,40" transparent="1" alphatest="on" />
		<ePixmap pixmap="skin_default/buttons/yellow.png" position="410,0" size="140,40" transparent="1" alphatest="on" />
		<ePixmap pixmap="skin_default/buttons/blue.png" position="410,360" size="140,40" transparent="1" alphatest="on" />
		<widget source="key_red" render="Label" position="0,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
		<widget source="key_green" render="Label" position="150,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
		<widget source="key_yellow" render="Label" position="300,0" zPosition="1" size="250,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
		<widget source="key_blue" render="Label" position="410,360" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
		<widget source="introduction" render="Label" position="0,42" size="550,74" zPosition="10" font="Regular;21" halign="center" valign="center" backgroundColor="#25062748" transparent="1" />
		<widget name="config" position="20,130" size="510,220" scrollbarMode="showOnDemand" />
		<widget name="consideration" position="0,356" font="Regular;20" halign="center" size="410,45" />
		</screen>"""

	def __init__(self, session, args = None):
		print "[ntpdate] - ntpdateSetup - __init__ start "	 + strftime("%H:%M:%S", localtime())
		Screen.__init__(self, session)
		self.session = session
		self.skin_path = plugin_path
		self["introduction"] = StaticText(_("Local: " + strftime("%H:%M", localtime()) + ", UTC: " + strftime("%H:%M", gmtime()) + "\nTransponder time is sometimes wrong! With internet connection, time can be fetched from a time server ..."))
		self["consideration"] = Label(_("Check/Set 'Time Zone' under 'User Preferences', 'Time Settings'!"))
		self.list = []		
		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"], 
		{
			"save": self.save, 
			"cancel": self.cancel, 
			"ok": self.save,
			"green": self.save,
			"yellow": self.set,
			"red": self.cancel,
			"blue": self.help 
		}, -2)
		self["key_yellow"] = StaticText(_("Set network time now"))
		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("Save"))		
		self["key_blue"]=StaticText(_("Help"))		
#		if not self.selectionChanged in self["config"].onSelectionChanged:
#			self["config"].onSelectionChanged.append(self.selectionChanged)
# 2011-04-14 crash due to KeyError: 'config'
		ConfigListScreen.__init__(self, self.list)
		self.createSetup()
		print "[ntpdate] - ntpdateSetup finished!"  + strftime("%H:%M:%S", localtime())

	def selectionChanged(self):
#		current = self.l.getCurrentSelection()
		self.session.open(MessageBox, "selectionChanged")
#		if current == self.:
#			self.session.openWithCallback(
#				self.SvrInputCallback,
#				InputBox,
#				title = _("Please enter name of a time server"),
#				text = ""
#			)

	def SvrInputCallback(self, res):
		if res:
			config.plugins.ntpdate.timeserver1=res
		
	def keyLeft(self):
		ConfigListScreen.keyLeft(self)
		self.createSetup()

	def keyRight(self):
		ConfigListScreen.keyRight(self)
		self.createSetup()

	def createSetup(self):
		print "[ntpdate] - ntpdateSetup - createSetup start ..."  + strftime("%H:%M:%S", localtime())
		self.list = [ ]
		self.oldrep=-1
		try:
			x=config.misc.useTransponderTime.value #exception, if not in config listed yet?
		except Exception, e:
			config.misc.useTransponderTime = ConfigYesNo(default = False)
		self.list.append(getConfigListEntry(_("Use Transponder Time"),config.misc.useTransponderTime))
		self.list.append(getConfigListEntry(_("Auto Run ntpdate at boot"), config.plugins.ntpdate.enable))
		if config.plugins.ntpdate.enable.value == True:
			self.list.append(getConfigListEntry(_("  Delay 1st execution [s]"), config.plugins.ntpdate.delay1))
			self.oldrep=config.plugins.ntpdate.repeat
			self.list.append(getConfigListEntry(_("  Repetition (0 = off) [h]"), self.oldrep))
			self.list.append(getConfigListEntry(_("  Notification Message (success)"), config.plugins.ntpdate.notify))
			self.list.append(getConfigListEntry(_("  Message Show Time [s]"), config.plugins.ntpdate.showtime))
		self.list.append(getConfigListEntry(_("Server Timeout [s]"), config.plugins.ntpdate.timeout))
		self.list.append(getConfigListEntry(_("Time Server 1"), config.plugins.ntpdate.timeserver1))
		if config.plugins.ntpdate.timeserver1.value != "":
			self.list.append(getConfigListEntry(_("Time Server 2"), config.plugins.ntpdate.timeserver2))
			if config.plugins.ntpdate.timeserver2.value != "":
				self.list.append(getConfigListEntry(_("Time Server 3"), config.plugins.ntpdate.timeserver3))		
		self["config"].list = self.list
		self["config"].l.setList(self.list)

	def save(self):
		print "[ntpdate] save"  + strftime("%H:%M:%S", localtime())
		for x in self["config"].list:
			x[1].save()		
#		if config.plugins.ntpdate.enable.value==True:
#			try:
#				f = open("/etc/rc.d/ntpdate", 'w')
#				f.write("#!/bin/sh\n");
#				string = "%s/ntpdate -b -t %d %s" %(plugin_path, config.plugins.ntpdate.timeout.value, config.plugins.ntpdate.timeserver1.value)
#				if config.plugins.ntpdate.timeserver2.value != "":
#					string = string + " " + config.plugins.ntpdate.timeserver2.value
#				if config.plugins.ntpdate.timeserver3.value != "":
#					string = string + " " +config.plugins.ntpdate.timeserver3.value
#				f.write(string + "\n")
#				f.close()
#				cmd = "chmod 775 /etc/rc.d/ntpdate"
#				res2 = popen(cmd).read()
#			print "[ntpdate] /etc/rc.d/ntpdate not created"
#			except IOError:
#				Notifications.AddNotification(MessageBox, _("Can't create /etc/rc.d/ntpdate"), type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)
		if config.plugins.ntpdate.enable.value==False:
			try:
				remove("/etc/rc.d/ntpdate") #exception, if file does not exist
				print "[ntpdate] /etc/rc.d/ntpdate removed"
			except IOError:
				print "[ntpdate] there was no /etc/rc.d/ntpdate to remove. Or remove failed due to other reason"
		self.checkClose()

	def help(self):
		self.session.open(Console,_("Showing readme.txt"),["cat %s%s" % (plugin_path, g_readme)])

	def cancel(self):
		print "[ntpdate] cancel"  + strftime("%H:%M:%S", localtime())
		for x in self["config"].list:
			x[1].cancel()
		self.close()	

	def checkClose(self):
		global mytimer
		if self.oldrep>0:
			mytimer.stop()
		if not config.misc.useTransponderTime.value and not config.plugins.ntpdate.enable.value:
			self.session.openWithCallback(self.cbClose, MessageBox,_("Neither Transponder nor Internet are active!\nAre you sure?"), MessageBox.TYPE_YESNO)
		else:
			if config.plugins.ntpdate.enable.value:
				x = config.plugins.ntpdate.repeat.value
				if ((x>0) and (x!=self.oldrep)):
					mytimer.start(x*3600, True)
			self.close()

	def cbClose(self, answer):
		if answer:
			self.close()

	def set(self):
		if config.plugins.ntpdate.timeserver1 == "":
			self.session.open(MessageBox, _("no timeserver configured!"))
		else:
			before = "Before: Local=" + strftime("%H:%M", localtime()) +", UTC=" + strftime("%H:%M", gmtime())
			cmd = "%s%s %s %s %s %s" % (plugin_path, "/ntpdate -b -t ", config.plugins.ntpdate.timeout.value, config.plugins.ntpdate.timeserver1.value, config.plugins.ntpdate.timeserver2.value, config.plugins.ntpdate.timeserver3.value)
			res = popen(cmd).read()
			if res == "":
				cmd = "ls -l %s%s" % (plugin_path, "/ntpdate")
				res = popen(cmd).read()
				if res[3]!="x":
					cmd = "chmod 755 %s%s" % (plugin_path, "/ntpdate")
					res = popen(cmd).read()
					self.session.open(MessageBox, _("ntpdate problem: attributes for ntpdate have not been correct! Fixed now! Try again!\n%s" % res), MessageBox.TYPE_INFO)
				else:
				  self.session.open(MessageBox, _("ntpdate problem: Internet connection ok? Time server ok?"), MessageBox.TYPE_INFO)
			else:
				self.session.open(MessageBox, _(before + "\n\nntpdate done! " + res + "\nAfter: Local=" + strftime("%H:%M", localtime()) + ", UTC=" + strftime("%H:%M", gmtime())), type=MessageBox.TYPE_INFO)

# end class ntpdateSetup


def main(session, **kwargs):
	session.open(ntpdateSetup)

def autostart2(reason, **kwargs):
	global mytimer
	print "[ntpdate] - Autostartup2 (WHERE_SESSIONSTART)" + plugin_path
#	if "session" in kwargs:
	if kwargs.has_key("session"):
		session = kwargs["session"]
		mytimer.savesession(session)

# 1st we have WHERE_AUTOSTART, but without an session, a few seconds later the WHERE_SESSIONSTART

def autostart(reason, **kwargs):
	global mytimer
	if reason == 0: #startup
		print "[ntpdate] - Autostartup (WHERE_AUTOSTART) " + plugin_path
		if config.plugins.ntpdate.enable.value:
			if config.plugins.ntpdate.delay1==0:
				print "[ntpdate] autostart NtpCall " + strftime("%H:%M:%S", localtime())
				before = "Before: Local=" + strftime("%H:%M", localtime()) +", UTC=" + strftime("%H:%M", gmtime())
				cmd = "%s%s %s %s %s %s" % (plugin_path, "/ntpdate -b -t ", config.plugins.ntpdate.timeout.value, config.plugins.ntpdate.timeserver1.value, config.plugins.ntpdate.timeserver2.value, config.plugins.ntpdate.timeserver3.value)
				res = popen(cmd).read()
				if res == "":
					cmd = "ls -l %s%s" % (plugin_path, "/ntpdate")
					res2 = popen(cmd).read()
					if res2[3]!="x":
						cmd = "chmod 755 %s%s" % (plugin_path, "/ntpdate")
						res2 = popen(cmd).read()
						Notifications.AddNotification(MessageBox, _("ntpdate problem: attributes for ntpdate have been not correct! Fixed now! Next time should be ok!\n%s" % res), type=MessageBox.TYPE_INFO)
					else:
						Notifications.AddNotification(MessageBox, "Add-In NtpDate AutoStart failed to get time from Internet!", type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)
				elif config.plugins.ntpdate.notify.value:
					Notifications.AddNotification(MessageBox, before + "\n\nntpdate " + res + "\nAfter: Local=" + strftime("%H:%M", localtime()) +", UTC=" + strftime("%H:%M", gmtime()) , type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)
				x = config.plugins.ntpdate.repeat.value
				if (x>0):
					print "[ntpdate] - next Sync via Timer in [h]:", x
					mytimer.start(x*3600, True)
			else: #with delay
#				try:
				x = config.plugins.ntpdate.delay1.value
				mytimer.start(x, True)
#				except:
#					print "[ntpdate] error starting timer!"
#					Notifications.AddNotification(MessageBox, "ntpdate timerstart failed" , type=MessageBox.TYPE_INFO, timeout=config.plugins.ntpdate.showtime.value)

# this helps for the EPG problem: (see usr\local\lib\enigm2\python\mytest.py)
			eDVBLocalTimeHandler.getInstance().setUseDVBTime(True)
	elif reason == 1:
		print "[ntpdate] - Stop Timer (autostart reason 1)"
		mytimer.stop()
		mytimer = None


def Plugins(path, **kwargs):
  global plugin_path
  plugin_path = path
  return [
    PluginDescriptor(name="ntpdate", description="Set DateTime via Internet " + g_pluginversion, where = PluginDescriptor.WHERE_PLUGINMENU, icon = "plugin.png", fnc=main),
    PluginDescriptor(where = PluginDescriptor.WHERE_AUTOSTART, fnc = autostart),
    PluginDescriptor(where = PluginDescriptor.WHERE_SESSIONSTART, fnc = autostart2)
    ]
